/* VERSION 2: VIDEO WITH TEXT CHAT  
 * =====================================================
 * Last updated: June 28, 2007.
 * 
 * Features:
 * - Simple two-way video conferencing 
 * - Text Chat
 * 
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace VideoChat
{
    public partial class Form1 : Form
    {
        // variable declarations
        EasyImages.CameraClient wc;
        GroupLab.GTNetworking.SharedDictionary SD;        
        GroupLab.GTNetworking.Subscription imageSubscription;
        GroupLab.GTNetworking.Subscription pingSubscription;
        GroupLab.GTNetworking.Subscription textSubscription;        
        
        /* Thread handling: Need this block of code to make sure code runs from the main windows form, and not any other thread */
        public delegate void SetControlImage(PictureBox icontrol, Bitmap image); // delegate: pointer to a function
        public void RecievedImageToControl(PictureBox icontrol, Image image)
        {
            /* Control.InvokeRequired -- gets a value indicating whether the caller must call an invoke method when making method 
             * calls to the control because the caller is on a different thread than the one the control was created on. 
             * Control.BeginInvoke() -- executes a delegate asynchronously on the thread that the control's underlying handle was created on.
             */             
            if (icontrol.InvokeRequired) 
            {
                SetControlImage theDelegate = new SetControlImage(RecievedImageToControl);
                this.BeginInvoke(theDelegate, new object[] { icontrol, image });                
            }
            else
                icontrol.Image = image;
        }//RecievedImageToControl

        public Form1()
        {
            Control.CheckForIllegalCrossThreadCalls = false;
            InitializeComponent();

            // camera declarations
            wc = new EasyImages.CameraClient("DefaultCamera");            
            wc.ReceivedFrame += new EasyImages.CameraClient.ReceivedFrameEventHandler(client_ReceivedFrame);
            wc.Start();
            
            // shared dictionary and subscription declarations
            SD = new GroupLab.GTNetworking.SharedDictionary();
            pingSubscription = new GroupLab.GTNetworking.Subscription();
            imageSubscription = new GroupLab.GTNetworking.Subscription();
            textSubscription = new GroupLab.GTNetworking.Subscription();
        }//Form1

        private void client_ReceivedFrame(object sender, EasyImages.CameraEventArgs e)
        {
            RecievedImageToControl(this.mirrorDisplay, e.Bitmap); 
            //this.MirrorDisplay.Image = e.Bitmap;
        }//client_ReceivedFrame

        private void ConnectionButton_Click(object sender, EventArgs e)
        {
            if (this.connectionButton.Text == "Connect!")
            {
                string url = "tcp://localhost:hello";

                if (DialogResult.OK == GroupLab.GTNetworking.UrlPrompt.Show(ref url))
                {
                    this.SD.Url = url;                    
                    this.SD.Opened += new EventHandler(SD_Opened);
                    this.SD.Closed +=  new EventHandler(SD_Closed);
                    this.SD.Open(); // start server or client
                    this.connectionButton.Text = "Disconnect!";
                }
            }
            else
            {
                this.connectionButton.Text = "Connect!";
                this.SD.Close(); // close the connection with the shared dictionary
            }
        }//ConnectionButton_Click

        private void SD_Opened(object sender, EventArgs e)
        {
            // imageSubscription
            this.imageSubscription.BeginInit();
            this.imageSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
                this.imageSubscription.Pattern = "/ServerImage";
            else
                this.imageSubscription.Pattern = "/ClientImage";
            this.imageSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(imageSubscription_Notified);
            this.imageSubscription.EndInit();

            // pingSubscription
            this.pingSubscription.BeginInit();
            this.pingSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
                this.pingSubscription.Pattern = "/ServerPing";
            else
                this.pingSubscription.Pattern = "/ClientPing";            
            this.pingSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(pingSubscription_Notified);
            this.pingSubscription.EndInit();

            // send the first frame
            takeSnap();

            // textSubscription
            this.textSubscription.BeginInit();
            this.textSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
                this.textSubscription.Pattern = "/ServerTextMessage";
            else
                this.textSubscription.Pattern = "/ClientTextMessage";            
            this.textSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(textSubscription_Notified);
            this.textSubscription.EndInit();
        }//SD_Opened

        private void SD_Closed(object sender, System.EventArgs args)
        {
        }//SD_Closed

        private void takeSnap()
        {
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
            {
                this.connectionTypeLabel.Text = "Connection Type: CLIENT";
                this.SD["/ClientImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);
            }
            else if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.connectionTypeLabel.Text = "Connection Type: SERVER";
                this.SD["/ServerImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);
            }

        }//takeSnap

        void imageSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)//imageSubscription_Notified
        {
            EasyImages.JPEG j = e.Value as EasyImages.JPEG;
            if (j != null)
            {
                RecievedImageToControl(this.videoDisplay, j.GetImage());
                //this.VideoDisplay.Image = j.GetImage();

            }
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
            {
                this.SD["/ClientPing"] = "ping!"; 
            }
            else if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.SD["/ServerPing"] = "ping!";
            }
        }//imageSubscription_Notified

        private void pingSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            takeSnap(); //if got, then send the next frame
        }//pingSubscription_Notified

        private void textSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            this.textLabel.Text = e.Value.ToString();
        }//textSubscription_Notified

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // if user chooses to close the form manually, do so in an elegant fashion (stop camera and close dictionary)
            wc.Stop();
            if (SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {
                SD.Close();
            }
        }//Form1_FormClosing

        private void chatBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Client)
            {
                this.SD["/ClientTextMessage"] = chatTextBox.Text + e.KeyChar.ToString();
            }
            else if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.SD["/ServerTextMessage"] = chatTextBox.Text + e.KeyChar.ToString();
            }
            if (e.KeyChar == '\r')
            {
                chatTextBox.Text = "";
            }
        }//chatBox_KeyPress

    }// partial class Form1
}// namespace VideoChat